package kal.base.jdf.com;

/**
 * @(#)
 * Copyright 1999-2000 by  LG-EDS Systems, Inc.,
 * Information Technology Group, Application Architecture Team, 
 * Application Intrastructure Part.
 * 236-1, Hyosung-2dong, Kyeyang-gu, Inchun, 407-042, KOREA.
 * All rights reserved.
 * 
 * NOTICE !      You can copy or redistribute this code freely, 
 * but you should not remove the information about the copyright notice 
 * and the author.
 * 
 * @author  WonYoung Lee, wyounglee@lgeds.lg.co.kr.
 */
import java.lang.reflect.*;
import java.util.Enumeration;
import java.util.Vector;
 
public final class Utility {

	/**
	 * Don't let anyone instantiate this class
	 */
	private Utility() {}

	/**
	 * Object[]  . Object Array  (clone)Ͽ
	 * ο Instance  ݴϴ.
	 *
	 * @param objects java.lang.Object[]
	 * @return java.lang.Object[]
	 *
	 * @see clone(java.lang.Object)
	 * @see clone(java.lang.Vector)
	 */
	public static Object[] clone(Object[] objects)
	{
		int length = objects.length;
		Class c = objects.getClass().getComponentType();
		Object array = Array.newInstance(c, length);

		for(int i=0;i<length;i++){
			Array.set(array, i, Utility.clone(objects[i]));
		}
		return (Object[])array;
	}

	/**
	 * Object  . Ϲ <code>java.lang.Object.clone()</code> Լ
	 *  Ͽ Object ϸ Object ִ Primitive type  Object
	 * field  Ǵ  ƴ϶  Object reference
	 *  ȴ.<br>
	 * ׷  Method ϸ  field  Object  (clone)Ͽ
	 * ش.
	 *
	 * @param object java.lang.Object
	 * @return java.lang.Object
	 *
	 * @see clone(java.lang.Object[])
	 * @see clone(java.lang.Vector)
	 */
	public static Object clone(Object object)
	{
		Class c = object.getClass();
		Object newObject = null;
		try {
			newObject = c.newInstance();
		}
		catch(Exception e ){
			return null;
		}

		Field[] field = c.getFields();
		for (int i=0 ; i<field.length; i++) {
			try {
				Object f = field[i].get(object);
				field[i].set(newObject, f);
			}
			catch(Exception e){
			}
		}
		return newObject;
	}

	/**
	 * Vector  . Ϲ Vector Object clone() ϸ
	 * Vector Element Object  Ǵ  ƴ϶
	 *  Object  reference  Ǳ   Element Object
	 * referenceϴ Vector ϰ ȴ. ׷  method ϸ
	 * Vector  Element Object  Ͽ ش.
	 *
	 * @param objects java.util.Vector
	 * @return java.util.Vector
	 *
	 * @see clone(java.lang.Object)
	 * @see clone(java.lang.Object[])
	 */
	public static Vector clone(Vector objects)
	{
		Vector newObjects = new Vector();
		Enumeration e = objects.elements();
		while(e.hasMoreElements()){
			Object o = e.nextElement();
			newObjects.addElement(Utility.clone(o));
		}
		return newObjects;
	}

	/**
	 * Entity Class null string field ʱȭ.
	 * <p>
	 * Entity class  ִ java.lang.String field DB Column
	 *   ִ 찡 . ̷ Entity Field Ư GUI Ư
	 * TextFiled ǥǾ ϴ 쵵 .   String Filed null
	 *   ˻縦 Ѵٴ     ƴ  .
	 * <p>
	 *  method  Object  ִ  java.lang.String field  
	 * null   field ̰ 0  blank string("") ʱȭ ش.
	 * <p>
	 *
	 * <xmp>
	 * Sample Code:
	 * public java.util.Vector selectAll() throws Exception
	 * {
	 *  java.util.Vector list = new Vector();
	 * 	Statement stmt = null;
	 * 	ResultSet rs =null;
	 * 	try{
	 * 		stmt = conn.createStatement();
	 * 		String query = "select " +
	 * 			"id, " + 
	 * 			"name, " + 
	 * 			"desc " + 
	 * 			"from THE10 " +
	 * 			"order by id ";
	 * 
	 * 		rs = stmt.executeQuery(query);
	 * 
	 * 		while ( rs.next() ) {
	 * 			AdminAuth entity = new AdminAuth();
	 * 			entity.id = rs.getString("id");
	 * 			entity.name = rs.getString("name");
	 * 			entity.desc = rs.getString("desc");
	 * 			Utility.fixNull(entity);
	 * 			list.addElement(entity);
	 * 		}
	 * 	}
	 * 	finally {
	 * 		try{rs.close();}catch(Exception e){}
	 * 		try{stmt.close();}catch(Exception e){}
	 * 	}
	 * 	return list;
	 * }
	 *</xmp
	 *
	 * @param java.lang.Object Object public java.lang.String  
	 *        member variable  ش.
	 *
	 * @see fixNullAll(java.lang.Object)
	 * @see fixNullAndTrim(java.lang.Object)
	 * @see fixNullAndTrimAll(java.lang.Object)
	 * @author WonYoung Lee, wyounglee@lgeds.lg.co.kr
	 */
	public static void fixNull(Object o)
	{
		if ( o == null ) return;
		
		Class c = o.getClass();
		if ( c.isPrimitive() ) return;
		
		Field[] fields = c.getFields();
		for (int i=0 ; i<fields.length; i++) {
			try {
				Object f = fields[i].get(o);
				Class fc = fields[i].getType();
				
				if ( fc.getName().equals("java.lang.String") ) {
					if ( f == null ) fields[i].set(o, "");
					else	fields[i].set(o, f);
				}
			}
			catch(Exception e){
			}
		}
	}

	/**
	 * Entity Class  null string field ʱȭ.
	 * <p>
	 * fixNull()    ϴµ, java.lang.String field Ӹ ƴ϶
	 * Member   Array, Object    i  String
	 * blank string("")  ش.<br>
	 *  String  trim() ش.<br>
	 *  Array, Vector null  Instanceȭ  ʴ´.
	 *
	 * <p>
	 *  Ǵ ŭ, θ ڽİ   reference  
	 *  ȵȴ. Stack Overflow  JVM  ̴.
	 *
	 *
	 * @param java.lang.Object Object public String Ӹ ƴ϶, Object[], Vector 
	 *         public Object Member Variable  ش.
	 *
	 * @see fixNull(java.lang.Object)
	 * @see fixNullAndTrim(java.lang.Object)
	 * @see fixNullAndTrimAll(java.lang.Object)
	 *
	 * @author 
	 */
	public static void fixNullAll(Object o)
	{
		if ( o == null ) return;
		
		Class c = o.getClass();
		if ( c.isPrimitive() ) return;
		
		if( c.isArray() ) {
			int length = Array.getLength(o);
			for(int i=0; i<length ;i++){
				Object element = Array.get(o, i);
				Utility.fixNullAll(element);
			}
		} 
		else {
			Field[] fields = c.getFields();
			for (int i=0 ; i<fields.length; i++) {
				try {
					Object f = fields[i].get(o);
					Class fc = fields[i].getType();
					if ( fc.isPrimitive() ) continue;
					if ( fc.getName().equals("java.lang.String") ) {
						if ( f == null ) fields[i].set(o, "");
						else	fields[i].set(o, f);
					} 
					else if ( f != null ) {
						Utility.fixNullAll(f);
					}
					else {} // Some Object, but it's null.
				}
				catch(Exception e) {
				}
			}
		}
	}

	/**
	 * Entity Class null string field ʱȭ &amp; trim().
	 * <p>
	 * Entity class  ִ java.lang.String field DB Column
	 *   ִ 찡 . ̷ Entity Field Ư GUI Ư
	 * TextFiled ǥǾ ϴ 쵵 .   String Filed null
	 *   ˻縦 Ѵٴ     ƴ  .
	 * <p>
	 *  method  Object  ִ  java.lang.String field  
	 * null   field ̰ 0  blank string("") ʱȭ ش.
	 *  null ƴ  String ԵǾ   trim()
	 * ش.
	 * <p>
	 *   trim() Լ java.lang.String  trim()  ʾҴ.
	 *
	 * <xmp>
	 * Sample Code:
	 * public java.util.Vector selectAll() throws Exception
	 * {
	 *  java.util.Vector list = new Vector();
	 * 	Statement stmt = null;
	 * 	ResultSet rs =null;
	 * 	try{
	 * 		stmt = conn.createStatement();
	 * 		String query = "select " +
	 * 			"id, " + 
	 * 			"name, " + 
	 * 			"desc " + 
	 * 			"from THE10 " +
	 * 			"order by id ";
	 * 
	 * 		rs = stmt.executeQuery(query);
	 * 
	 * 		while ( rs.next() ) {
	 * 			AdminAuth entity = new AdminAuth();
	 * 			entity.id = rs.getString("id");
	 * 			entity.name = rs.getString("name");
	 * 			entity.desc = rs.getString("desc");
	 * 			Utility.fixNull(entity);
	 * 			list.addElement(entity);
	 * 		}
	 * 	}
	 * 	finally {
	 * 		try{rs.close();}catch(Exception e){}
	 * 		try{stmt.close();}catch(Exception e){}
	 * 	}
	 * 	return list;
	 * }
	 *</xmp
	 *
	 * @param java.lang.Object Object public java.lang.String  
	 *        member variable  ش.
	 *
	 * @see fixNull(java.lang.Object)
	 * @see fixNullAll(java.lang.Object)
	 * @see fixNullAndTrimAll(java.lang.Object)
	 * @see trim(String)
	 * @author WonYoung Lee, wyounglee@lgeds.lg.co.kr
	 */
	public static void fixNullAndTrim(Object o)
	{
		if ( o == null ) return;
		
		Class c = o.getClass();
		if ( c.isPrimitive() ) return;
		
		Field[] fields = c.getFields();
		for (int i=0 ; i<fields.length; i++) {
			try {
				Object f = fields[i].get(o);
				Class fc = fields[i].getType();
				if ( fc.getName().equals("java.lang.String") ) {
					if ( f == null ) fields[i].set(o, "");
					else {
						String item = Utility.trim( (String)f );
						fields[i].set(o, item);
					}
				}
			}
			catch(Exception e){
			}
		}
	}

	/**
	 * Entity Class  null string field ʱȭ  &amp; trim().
	 * <p>
	 * fixNull()    ϴµ, java.lang.String field Ӹ ƴ϶
	 * Member   Array, Object    i  String
	 * blank string("")  ش.<br>
	 *  String  trim() ش.<br>
	 *  Array, Vector null  Instanceȭ  ʴ´.
	 *
	 * <p>
	 *  Ǵ ŭ, θ ڽİ   reference  
	 *  ȵȴ. Stack Overflow  JVM  ̴.
	 *
	 *
	 * @param java.lang.Object Object public String Ӹ ƴ϶, Object[], Vector 
	 *         public Object Member Variable  ش.
	 *
	 * @see fixNull(java.lang.Object)
	 * @see fixNullAll(java.lang.Object)
	 * @see fixNullAndTrim(java.lang.Object)
	 * @see trim(String)
	 *
	 * @author , ̿
	 */
	public static void fixNullAndTrimAll(Object o)
	{
		if ( o == null ) return;
		
		Class c = o.getClass();
		if ( c.isPrimitive() ) return;
		
		if( c.isArray() ) {
			int length = Array.getLength(o);
			for(int i=0; i<length ;i++){
				Object element = Array.get(o, i);
				Utility.fixNullAndTrimAll(element);
			}
		} 
		else {
			Field[] fields = c.getFields();
			for (int i=0 ; i<fields.length; i++) {
				try {
					Object f = fields[i].get(o);
					Class fc = fields[i].getType();
					if ( fc.isPrimitive() ) continue;
					if ( fc.getName().equals("java.lang.String") ) {
						if ( f == null ) fields[i].set(o, "");
						else {
							String item = Utility.trim( (String)f );
							fields[i].set(o, item);
						}
					} 
					else if ( f != null ) {
						Utility.fixNullAndTrimAll(f);
					}
					else {} // Some Object, but it's null.
				}
				catch(Exception e) {
				}
			}
		}
	}

	/**
	 * 
	 * @param e java.lang.Throwable
	 */
	public static String getStackTrace(Throwable e) {
		java.io.ByteArrayOutputStream bos = new java.io.ByteArrayOutputStream();
		java.io.PrintWriter writer = new java.io.PrintWriter(bos);
		e.printStackTrace(writer);
		writer.flush();
		return bos.toString();
	}

	/**
	 * Remove special white space from both ends of this string. 
	 * <p>
	 * All characters that have codes less than or equal to 
	 * <code>'&#92;u0020'</code> (the space character) are considered to be 
	 * white space. 
	 * <p>
	 * java.lang.String trim()  Ϲ white space ¥ 
	 * ƴ϶   Ư blank © ش.<br>
	 *  ҽ IBM HOST Ÿ ְ   ϰ ߾.
	 * Ϲ    ̴.
	 *
	 * @param  java.lang.String
	 * @return trimed string with white space removed
	 *         from the front and end.
	 * @author WonYoung Lee, wyounglee@lgeds.lg.co.kr
	 */
	public static String trim(String s) {
		int st = 0;
		char[] val = s.toCharArray();
		int count = val.length;
		int len = count;

		while ((st < len) && ((val[st] <= ' ') || (val[st] == '') ) )   st++;
		while ((st < len) && ((val[len - 1] <= ' ') || (val[len-1] == '')))  len--;
		
		return ((st > 0) || (len < count)) ? s.substring(st, len) : s;
	}
	
	public static String removeComma(String s) {
		if ( s == null ) return null;
		if ( s.indexOf(",") != -1 ) {
			StringBuffer buf = new StringBuffer();
			for(int i=0;i<s.length();i++){
				char c = s.charAt(i);
				if ( c != ',') buf.append(c);
			}
			return buf.toString();
		}
		return s;
	}
	
	public static String toTrimStrValue ( String val ) {
		String temp = null;
		
		temp  = (val == null) ? "" : val.trim();
		
		return temp;
	}
	
	public static long toLongValue ( String val ) {
		long temp = 0;
		
		val  = (val == null) ? "" : removeComma(val.trim());
		if (val.equals("") ) val = "0";
		temp = Long.parseLong(val);
		
		return temp;
	}
	
	public static double toDoubleValue ( String val ) {
		double temp = 0;
		
		val  = (val == null) ? "" : removeComma(val.trim());
		if (val.equals("") ) val = "0";
		temp = Double.parseDouble(val);
		
		return temp;
	}
	
}