package kal.base.jdf.ejb;

/**
 * 
 * @(#) EJBFinder.java
 * Copyright 1999-2000 by  LG-EDS Systems, Inc.,
 * Information Technology Group, Application Architecture Team, 
 * Application Intrastructure Part.
 * 236-1, Hyosung-2dong, Kyeyang-gu, Inchun, 407-042, KOREA.
 * All rights reserved.
 *  
 * NOTICE !      You can copy or redistribute this code freely, 
 * but you should not remove the information about the copyright notice 
 * and the author.
 *  
 * @author  WonYoung Lee, wyounglee@lgeds.lg.co.kr.
 */

import java.util.Hashtable;

import javax.ejb.EJBHome;
import javax.ejb.EJBObject;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;

import java.lang.reflect.Method;

public final class EJBFinder {
	/**
	 * Don't let anyone instantiate this class
	 */
	private EJBFinder() {}

	/**
	 * lookup(contextInfo, name, null);
	 * @return EJBObject remote interface.
	 * @param name The string representation of the Bean object.
	 */
	public static EJBObject lookup(Hashtable contextInfo, String name) throws Exception {
		return lookup(contextInfo, name, null);
	}

	/**
	 * 
	 * @return EJBObject remote interface.
	 * @param name The string representation of the Bean object.
	 * @param args the array of arguments for calling EJBHome.create( args ).
	 *               if args is null, it means that just invoke create().
	 */
	public static EJBObject lookup(Hashtable contextInfo, String name, Object[] args) throws Exception {

		InitialContext initialContext =  new InitialContext(contextInfo);
		return lookup(initialContext, name, args);
	}

	/**
	 * lookup(contextInfo, name, null);
	 * @return EJBObject remote interface.
	 * @param name The string representation of the Bean object.
	 */
	public static EJBObject lookup(InitialContext initialContext, String name) throws Exception {
		return lookup(initialContext, name, null);
	}

	/**
	 * 
	 * @return EJBObject remote interface.
	 * @param name The string representation of the Bean object.
	 * @param args the array of arguments for calling EJBHome.create( args ).
	 *               if args is null, it means that just invoke create().
	 */
	public static EJBObject lookup(InitialContext initialContext, String name, Object[] args) throws Exception {
		
		Object home = initialContext.lookup(name);
		
		EJBHome ejbHome = (EJBHome) PortableRemoteObject.narrow ((org.omg.CORBA.Object) home, EJBHome.class);
		
		//Class c = home.getClass();
		Class c = ejbHome.getClass();		
		
		Class[] argsClasses = null;
		if ( args != null ) {
			int length = args.length;
			argsClasses = new Class[length];
			for(int i=0; i<length; i++) {
				argsClasses[i] = args[i].getClass();
			}
		}
		Method method = c.getDeclaredMethod("create", argsClasses);
		EJBObject bean = (EJBObject) method.invoke(home, args);
		return bean;
	}

	/**
	 * by chkim
	 */
	public static EJBObject lookup(String name) throws Exception {

		return lookup(name, null);
	}

	/**
	 * by chkim
	 */
	public static EJBObject lookup(String name, Object[] args) throws Exception {

		Hashtable props = new Hashtable();
		//props.put(Context.PROVIDER_URL, "IIOP://localhost:900/");  // update - chkim
		props.put("java.naming.factory.url", "IIOP://localhost:2809/");  // update - juping
		//props.put(Context.INITIAL_CONTEXT_FACTORY, "com.ibm.ejs.ns.jndi.CNInitialContextFactory");
		props.put("java.naming.factory.initial", "com.ibm.websphere.naming.WsnInitialContextFactory");
		InitialContext initialContext = new InitialContext(props);

		return lookup(initialContext, name, args);
	}
}